// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.

#pragma once

#include <unordered_map>

#include "core/common/common.h"
#include "core/platform/ort_mutex.h"
#include "core/providers/rocm/rocm_pch.h"

namespace onnxruntime {

using CudaGraphAnnotation_t = int;
using CudaGraphSet_t = std::unordered_map<CudaGraphAnnotation_t, hipGraphExec_t>;

constexpr CudaGraphAnnotation_t kCudaGraphAnnotationSkip = -1;
constexpr CudaGraphAnnotation_t kCudaGraphAnnotationDefault = 0;

struct CudaGraphSet {
  CudaGraphSet() {};
  ~CudaGraphSet();

  void Clear();
  bool Contains(CudaGraphAnnotation_t rocm_graph_annotation_id) const;
  void Put(CudaGraphAnnotation_t rocm_graph_annotation_id, hipGraphExec_t graph_exec);
  hipGraphExec_t Get(CudaGraphAnnotation_t rocm_graph_annotation_id) const;

 private:
  CudaGraphSet_t rocm_graphs_;
};

struct ROCMGraphManager {
  ROCMGraphManager() {};
  ROCMGraphManager(hipStream_t stream);
  ~ROCMGraphManager();

  void SetStream(hipStream_t stream);
  void CaptureBegin(CudaGraphAnnotation_t rocm_graph_annotation_id);
  void CaptureEnd(CudaGraphAnnotation_t rocm_graph_annotation_id);
  Status Replay(CudaGraphAnnotation_t rocm_graph_annotation_id);

  void Reset();

  bool IsGraphCaptureAllowedOnRun(CudaGraphAnnotation_t rocm_graph_annotation_id) const;
  bool IsGraphCaptured(CudaGraphAnnotation_t rocm_graph_annotation_id) const;

 private:
  CudaGraphSet rocm_graph_set_;
  CudaGraphAnnotation_t rocm_graph_annotation_id_ = kCudaGraphAnnotationDefault;

  hipStream_t stream_ = nullptr;  // Does not own the stream
};

using ROCMGraph = ROCMGraphManager;

}  // namespace onnxruntime
